package kamera;

import processing.core.*; 

import java.net.*;
import java.util.Enumeration;
import java.util.Collections;

import java.util.Properties;
import java.util.Set;
import java.util.Map;

//import org.apache.http.conn.util.InetAddressUtils;

//Um TIMEOFFSET.txt vom Downloadbereich zu holen. Achtung: Read/Wrizte-Permission auf Filesystem nötig.

/**
*  <h4>Info.java</h4>
*
*  Copyright (C) 2015 Guido Kramann<br/>
*  kramann@fh-brandenburg.de<br/>
*  http://www.kramann.info<br/>
*  <br/>
*  
*  Stellt Informationen über den Rechner zur Verfügung.             <br/>
*  U.a. das aktuelle Betriebssystem (Linux, Windows, Android, ...)  <br/>
*  Kann dazu genutzt werden, zwischen Plattform abhängigem Code     <br/>
*  automatisch umzuschalten.                                        <br/>
*  <br/>
*  ACHTUNG: Die Methoden isAndroid() und isLinux() müssen noch ausführlich getestet werden. <br/>
*  Es kann sein, dass die Art der Festlegung (Betriebssystem-Version enthält das Wort "generic") nicht immer funktioniert.<br/>
*/
public class Info
{
    public static long TIMEOFFSETMILLIS = 0;
    
    private String[] hex = {"0","1","2","3","4","5","6","7","8","9","a","b","c","d","e","f"};

    private String user_name;
    private String home_verzeichnis;
    private String betriebssystem;
    private String betriebssystem_architektur;
    private String betriebssystem_version;
    private String zeilenumbruch;
    private String ip = null;
    private String mac = null;

    public String getUserName() { return user_name; }
    public String getHomeVerzeichnis() { return home_verzeichnis; }
    public String getBetriebssystem() { return betriebssystem; }
    public String getBetriebssystemArchitektur() { return betriebssystem_architektur; }
    public String getBetriebssystemVersion() { return betriebssystem_version; }
    public String getZeilenumbruch() { return zeilenumbruch; }
    public String getIP() { return ip; }
    public String getMAC() { return mac; }

    
    public static long currentTimeMillis()
    {
        return System.currentTimeMillis() + TIMEOFFSETMILLIS;
    }
    
    /**
    *   Liefert Array mit zwei Spalten:<br/>
    *   Bezeichner | Wert
    */
    public String[][] getInfo()
    {
        return new String[][] {
                                  {"UserName",user_name},
                                  {"HomeVerzeichnis",home_verzeichnis},
                                  {"Betriebssystem",betriebssystem},
                                  {"BetriebssystemArchitektur",betriebssystem_architektur},
                                  {"BetriebssystemVersion",betriebssystem_version},
                                  {"Zeilenumbruch",zeilenumbruch},
                                  {"IP",ip},
                                  {"MAC",mac}
                              };
    }

    public void printInfo()
    {
        String[][] inf = getInfo();
        for(int i=0;i<inf.length;i++)
        {
            System.out.println(i+".: "+inf[i][0]+": "+inf[i][1]);
        }
    }

    /**
    *   Ausgabe aller Info-Daten auf dem Screen von Processing. <br/>
    *   Aufrufbeispiel aus einem Processing-Scetch heraus: <br/>
    *   Info info = new Info();<br/>
    *   info.showInfo(this,50,50);<br/>
    */
    public void showInfo(PApplet pap, int xoffset, int yoffset)
    {
        String[][] inf = getInfo();
        for(int i=0;i<inf.length;i++)
        {
            pap.text(i+".: "+inf[i][0]+": "+inf[i][1],xoffset,yoffset+30*i);
        }
    }

    public Info()
    {
        Properties prop = System.getProperties();
        Set<String> keys = prop.stringPropertyNames();
        Object[] keyso = keys.toArray();
        //for(int i=0;i<keyso.length;i++)
        //    println(keyso[i]);
        betriebssystem     = prop.getProperty("os.name");
        betriebssystem_architektur = prop.getProperty("os.arch");
        betriebssystem_version = prop.getProperty("os.version");
        home_verzeichnis   = prop.getProperty("user.home"); 
        user_name          = prop.getProperty("user.name"); 
        zeilenumbruch      = prop.getProperty("line.separator");   

        //bestimmeIPundMAC();

        ip = getLocalIpv4Address();

        System.out.println("ip="+ip);
        System.out.println("mac="+mac);
        
    }   

    public boolean isAndroid()
    {
        if(betriebssystem.equals("Linux") && !betriebssystem_version.contains("generic"))
            return true;
        else
            return false;
    }
    public boolean isLinux()
    {
        if(betriebssystem.equals("Linux") && betriebssystem_version.contains("generic"))
            return true;
        else
            return false;
    }


    /**
    *   Android-Code IP-Adresse holen
    */
	public String getLocalIpv4Address() 
        {
		try {
			for (Enumeration<NetworkInterface> networkInterfaceEnum = NetworkInterface
					.getNetworkInterfaces(); networkInterfaceEnum.hasMoreElements();) 
                        {
				NetworkInterface networkInterface = networkInterfaceEnum.nextElement();
				for (Enumeration<InetAddress> ipAddressEnum = networkInterface
						.getInetAddresses(); ipAddressEnum.hasMoreElements();) 
                                {
					InetAddress inetAddress = (InetAddress) ipAddressEnum
							.nextElement();
					//---Prüfen, ob es sich um eine Loopback-Adresse handelt,
					// dass es IPv4 ist---
                                        String iptest = ""+inetAddress;
					if (!inetAddress.isLoopbackAddress()
                                             //würde nur unter Android laufen:
                                             //&& InetAddressUtils.isIPv4Address(inetAddress.getHostAddress())
                                             //ersatzweise:
                                             && !iptest.contains(":")
                                           ) 
                                        {
                                                //gleich MAC-Adresse mit bestimmen:
                                                byte[] ha = networkInterface.getHardwareAddress();
                                                mac="";
                                                for(int k=0;k<ha.length;k++)
                                                {
                                                    int x = ha[k];
                                                    if(x<0)
                                                        x=256+x;
                                                    mac+=hex[x/16];
                                                    mac+=hex[x%16];
                                                    if(k<ha.length-1)
                                                        mac+=":";
                                                } 


						return inetAddress.getHostAddress();
					}
				}
			}
		} catch (SocketException ex) {
//			Log.e("getLocalIpv4Address", ex.toString());
		}
		return null;
	}

	/**
	 * Hilfsmethode, um zu bestimmen, welches Device "die kelinste IP" hat.
	 * Liefert bei 172.14.0.83 die 83<br/>
	 * Liefert bei 127.0.0.1 die 1<br/>
	 * usw.
	 */
	public int getNiedrigstenIPteil(String ipa)
	{
	     if(ipa!=null && ipa.length()>=7)
	     {
	         int end = ipa.length()-1;
	         int anzahl=0;
	         while(ipa.charAt(end)!='.' && end>0 )
	         {
	             end--;
	             anzahl++;
	         }    
	         String part = ipa.substring(end+1,end+1+anzahl);    
	         return (int)Double.parseDouble(part);
	     }
	     else
	     {
	         return Integer.MAX_VALUE;
	     }
	}
	
    /**
    *   Linux-Code IP- und MAC-Adresse  bestimmen
    */
    public void bestimmeIPundMAC()
    {
        try 
        {
            ip = "";
            Enumeration<NetworkInterface> ints = NetworkInterface.getNetworkInterfaces();
            for(NetworkInterface ni : Collections.list(ints))
            {
                 byte[] ha = ni.getHardwareAddress();
                 Enumeration<InetAddress> add = ni.getInetAddresses();
                 for(InetAddress ia : Collections.list(add))
                 {
                     String ip_neu = ""+ia.getHostAddress();
                     System.out.println("alternative ip: "+ip_neu);
                     if(!ip_neu.contains(":") && ip_neu.length()>ip.length())
                     {
                         ip = ip_neu;
                         mac="";
                         for(int k=0;k<ha.length;k++)
                         {
                             int x = ha[k];
                             if(x<0)
                                 x=256+x;
                             mac+=hex[x/16];
                             mac+=hex[x%16];
                             if(k<ha.length-1)
                                 mac+=":";
                         } 
                     }
                     System.out.println(""+ia.getHostAddress());
                 }
            }

//            if(mac==null)
//                ip = getLocalIpv4Address();
        } 
        catch (Exception e) 
        {

        }      
    }
}